# arch_is_concerned() tests
#
# Copyright (C) 2014, 2018, 2019  Patrick McDermott
#
# This file is part of the ProteanOS Development Kit.
#
# The ProteanOS Development Kit is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# The ProteanOS Development Kit is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the ProteanOS Development Kit.  If not, see
# <http://www.gnu.org/licenses/>.

set -eu

main()
{
	plan_ 24

	# Architecture: all
	command_ok_ \
		'HOST "amd64-linux-glibc" DOES NOT MATCH ARCH "all"' -- \
		not arch_is_concerned \
		'amd64-linux-glibc' \
		'all'
	command_ok_ \
		'HOST "i686-linux-glibc" DOES NOT MATCH ARCH "all"' -- \
		not arch_is_concerned \
		'i686-linux-glibc' \
		'all'
	command_ok_ \
		'HOST "all" DOES MATCH ARCH "all"' -- \
		arch_is_concerned \
		'all' \
		'all'

	# Architecture: any
	command_ok_ \
		'HOST "amd64-linux-glibc" DOES MATCH ARCH "any"' -- \
		arch_is_concerned \
		'amd64-linux-glibc' \
		'any'
	command_ok_ \
		'HOST "i686-linux-glibc" DOES MATCH ARCH "any"' -- \
		arch_is_concerned \
		'i686-linux-glibc' \
		'any'
	command_ok_ \
		'HOST "all" DOES NOT MATCH ARCH "any"' -- \
		not arch_is_concerned \
		'all' \
		'any'

	# Architecture: amd64-linux-glibc
	command_ok_ \
		'HOST "amd64-linux-glibc" DOES MATCH ARCH '$(: \
			)'"amd64-linux-glibc"' -- \
		arch_is_concerned \
		'amd64-linux-glibc' \
		'amd64-linux-glibc'
	command_ok_ \
		'HOST "i686-linux-glibc" DOES NOT MATCH ARCH '$(: \
			)'"amd64-linux-glibc"' -- \
		not arch_is_concerned \
		'i686-linux-glibc' \
		'amd64-linux-glibc'
	command_ok_ \
		'HOST "all" DOES NOT MATCH ARCH "amd64-linux-glibc"' -- \
		not arch_is_concerned \
		'all' \
		'amd64-linux-glibc'

	# Architecture: !amd64-linux-glibc
	command_ok_ \
		'HOST "amd64-linux-glibc" DOES NOT MATCH ARCH '$(: \
			)'"!amd64-linux-glibc"' -- \
		not arch_is_concerned \
		'amd64-linux-glibc' \
		'!amd64-linux-glibc'
	command_ok_ \
		'HOST "i686-linux-glibc" DOES MATCH ARCH '$(: \
			)'"!amd64-linux-glibc"' -- \
		arch_is_concerned \
		'i686-linux-glibc' \
		'!amd64-linux-glibc'
	command_ok_ \
		'HOST "all" DOES NOT MATCH ARCH "!amd64-linux-glibc"' -- \
		not arch_is_concerned \
		'all' \
		'!amd64-linux-glibc'

	# Architecture: amd64-linux-glibc cortexa8-linux-glibc
	command_ok_ \
		'HOST "amd64-linux-glibc" DOES MATCH ARCH '$(: \
			)'"amd64-linux-glibc cortexa8-linux-glibc"' -- \
		arch_is_concerned \
		'amd64-linux-glibc' \
		'amd64-linux-glibc cortexa8-linux-glibc'
	command_ok_ \
		'HOST "i686-linux-glibc" DOES MATCH ARCH '$(: \
			)'"amd64-linux-glibc cortexa8-linux-glibc"' -- \
		not arch_is_concerned \
		'i686-linux-glibc' \
		'amd64-linux-glibc cortexa8-linux-glibc'
	command_ok_ \
		'HOST "all" DOES MATCH ARCH '$(: \
			)'"amd64-linux-glibc cortexa8-linux-glibc"' -- \
		not arch_is_concerned \
		'all' \
		'amd64-linux-glibc cortexa8-linux-glibc'

	# Architecture: amd64-linux-glibc all
	command_ok_ \
		'HOST "amd64-linux-glibc" DOES MATCH ARCH '$(: \
			)'"amd64-linux-glibc all"' -- \
		arch_is_concerned \
		'amd64-linux-glibc' \
		'amd64-linux-glibc all'
	command_ok_ \
		'HOST "i686-linux-glibc" DOES NOT MATCH ARCH '$(: \
			)'"amd64-linux-glibc all"' -- \
		not arch_is_concerned \
		'i686-linux-glibc' \
		'amd64-linux-glibc all'
	command_ok_ \
		'HOST "all" DOES MATCH ARCH "amd64-linux-glibc all"' -D TODO --\
		arch_is_concerned \
		'all' \
		'amd64-linux-glibc all'

	# Architecture: amd64-any-any
	command_ok_ \
		'HOST "amd64-linux-glibc" DOES NOT MATCH ARCH "amd64-any-any"' -- \
		arch_is_concerned \
		'amd64-linux-glibc' \
		'amd64-any-any'
	command_ok_ \
		'HOST "i686-linux-glibc" DOES MATCH ARCH "amd64-any-any"' -- \
		not arch_is_concerned \
		'i686-linux-glibc' \
		'amd64-any-any'
	command_ok_ \
		'HOST "all" DOES NOT MATCH ARCH "amd64-any-any"' -- \
		not arch_is_concerned \
		'all' \
		'amd64-any-any'

	# Architecture: cortexa8-any-any i686-any-any
	command_ok_ \
		'HOST "amd64-linux-glibc" DOES NOT MATCH ARCH '$(: \
			)'"cortexa8-any-any i686-any-any"' -- \
		not arch_is_concerned \
		'amd64-linux-glibc' \
		'cortexa8-any-any i686-any-any'
	command_ok_ \
		'HOST "i686-linux-glibc" DOES MATCH ARCH '$(: \
			)'"cortexa8-any-any i686-any-any"' -- \
		arch_is_concerned \
		'i686-linux-glibc' \
		'cortexa8-any-any i686-any-any'
	command_ok_ \
		'HOST "all" DOES NOT MATCH ARCH '$(: \
			)'"cortexa8-any-any i686-any-any"' -- \
		not arch_is_concerned \
		'all' \
		'cortexa8-any-any i686-any-any'

	return 0
}
